/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) 2020 Loongson Technology Corporation Limited
 */
#ifndef _ASM_MODULE_H
#define _ASM_MODULE_H

#include <asm-generic/module.h>
#include <asm/orc_types.h>
#include <asm/inst.h>
#include <asm/loongarchregs.h>

#define LA_RELA_STACK_DEPTH 16

struct module;
u64 module_emit_plt_entry(struct module *mod, u64 val);

struct mod_section {
	struct elf64_shdr *shdr;
	int num_entries;
	int max_entries;
};

struct mod_arch_specific {
#ifdef CONFIG_UNWINDER_ORC
	unsigned int num_orcs;
	int *orc_unwind_ip;
	struct orc_entry *orc_unwind;
#endif
#ifdef CONFIG_DYNAMIC_FTRACE
	void *ftrace_trampolines[1 + CONFIG_DYNAMIC_FTRACE_WITH_REGS];
#endif
	struct mod_section plt;
	struct mod_section plt_idx;
};

struct plt_entry {
	u32 inst_lu12iw;
	u32 inst_lu32id;
	u32 inst_lu52id;
	u32 inst_jirl;
};

struct plt_idx_entry {
	unsigned long symbol_addr;
};

static inline struct plt_entry emit_plt_entry(u64 val, u64 plt, u64 plt_idx)
{
	u32 lu12iw, lu32id, lu52id, jirl;

	lu12iw = (0x0000000a << 25 | (((val >> 12) & 0xfffff) << 5) | REG_T1),
	lu32id = la64_insn_gen_lu32id(REG_T1, ADDR_IMM(val, LU32ID));
	lu52id = la64_insn_gen_lu52id(REG_T1, REG_T1, ADDR_IMM(val, LU52ID));
	jirl = la64_insn_gen_jirl(0, REG_T1, 0, (val & 0xfff));

	return (struct plt_entry) {
		lu12iw, lu32id, lu52id, jirl
	};
}

static inline struct plt_idx_entry emit_plt_idx_entry(u64 val)
{
	return (struct plt_idx_entry) {val};
}

static inline int get_plt_idx(u64 val, const struct mod_section *sec)
{
	struct plt_idx_entry *plt_idx = (struct plt_idx_entry *)sec->shdr->sh_addr;
	int i;

	for (i = 0; i < sec->num_entries; i++) {
		if (plt_idx[i].symbol_addr == val)
			return i;
	}
	return -1;
}

static inline struct plt_entry *get_plt_entry(u64 val,
				      const struct mod_section *sec_plt,
				      const struct mod_section *sec_plt_idx)
{
	struct plt_entry *plt = (struct plt_entry *)sec_plt->shdr->sh_addr;
	int plt_idx = get_plt_idx(val, sec_plt_idx);

	if (plt_idx >= 0)
		return plt + plt_idx;
	else
		return NULL;
}

#ifndef CONFIG_DYNAMIC_FTRACE
static inline int apply_ftrace_tramp(struct module *mod, void *addr, size_t size)
{
	return 0;
}
#else
extern int apply_ftrace_tramp(struct module *mod, void *addr, size_t size);
#endif

#define MODULE_PROC_FAMILY "LOONGARCH "

#ifdef CONFIG_32BIT
#define MODULE_KERNEL_TYPE "32BIT "
#elif defined CONFIG_64BIT
#define MODULE_KERNEL_TYPE "64BIT "
#endif

#define MODULE_ARCH_VERMAGIC \
	MODULE_PROC_FAMILY MODULE_KERNEL_TYPE

#endif /* _ASM_MODULE_H */
