#ifndef _ASM_SW64_CHIP_QEMU_IO_H
#define _ASM_SW64_CHIP_QEMU_IO_H

#include <asm/platform.h>

#define IO_BASE			(0x1UL << 47)
#define PCI_BASE		(0x1UL << 43)
#define PCI_64BIT_MEMIO		(0x1UL << 39)
#define PCI_IOR0_BASE		(0x2UL << 32)
#define PCI_IOR1_BASE		(0x3UL << 32)
#define PCI_RC_CFG		(0x5UL << 32)
#define PCI_EP_CFG		(0x3UL << 33)
#define PCI_LEGACY_IO		(0x1UL << 32)
#define PCI_MEM_UNPRE		0x0UL
#define PCI_32BIT_MEMIO		(0xc0000000UL)
#define PCI_32BIT_MEMIO_SIZE	(0x20000000UL)
#define PCI_LEGACY_IO_SIZE	(0x100000000UL)
#define PCI_64BIT_MEMIO_SIZE	(0x8000000000UL)

#define IO_RC_SHIFT		40
#define IO_NODE_SHIFT           44

/* MSIConfig */
#define MSICONFIG_VALID		(0x1UL << 63)
#define MSICONFIG_EN		(0x1UL << 62)
#define MSICONFIG_VECTOR_SHIFT	10

#define SW64_PCI_IO_BASE(m, n) \
	(IO_BASE | ((m) << IO_NODE_SHIFT) | PCI_BASE | ((n) << IO_RC_SHIFT))
#define SW64_IO_BASE(x)         (IO_BASE | ((x) << IO_NODE_SHIFT))

#define IO_MARK_BIT		47
#define CG_NUM			4

#define SW64_PCI0_BUS		0x0
#define PCI0_BUS		SW64_PCI0_BUS

#define MAX_NR_NODES		0x2
#define MAX_NR_RCS		0x6

#define SW64_PCI_DEBUG		0
#if SW64_PCI_DEBUG
#define PCIINFO(fmt, args...)	printk(fmt, ##args)
#else
#define PCIINFO(fmt, args...)
#endif

#define MCU_BASE		(0x3UL << 36)
#define INTPU_BASE		(0x2aUL << 32)
#define IO_NODE_SHIFT		44

#ifdef CONFIG_SUBARCH_C3A
#define INIT_CTL		((0x5UL << 36) | 0x680UL)
#define LONG_TIME		((0x5UL << 36) | 0x1180UL)
#endif

#ifdef CONFIG_SUBARCH_C3B
#define INIT_CTL		(MCU_BASE | 0x680UL)
#define LONG_TIME		(INTPU_BASE | 0x180UL)
#endif

/*-----------------------addr----------------------------------------------*/
/* MCU REG */
enum {
	CORE_ONLINE =		MCU_BASE | 0x780UL,
	CPU_ID =		MCU_BASE | 0x900UL,
	SI_FAULT_INT_EN =	MCU_BASE | 0x3200UL,
	MC_ONLINE =		MCU_BASE | 0x3780UL,
};

/* PIU IOR0 */
enum {
	PIUCONFIG0 =            0x0UL,
	EPDMABAR =		0x80UL,
	IOMMUSEGITEM0 =         0x100UL,
	IOMMUEXCPT_CTRL =       0x2100UL,
	MSICONFIG0 =		0x2200UL,
	DTBASEADDR =            0xb000UL,
	DTLB_FLUSHALL =         0xb080UL,
	DTLB_FLUSHDEV =         0xb100UL,
	PTLB_FLUSHALL =         0xb180UL,
	PTLB_FLUSHDEV =         0xb200UL,
	PTLB_FLUSHVADDR =       0xb280UL,
	PCACHE_FLUSHALL =       0xb300UL,
	PCACHE_FLUSHDEV =       0xb380UL,
	PCACHE_FLUSHPADDR =     0xb400UL,
	TIMEOUT_CONFIG =        0xb480UL,
	IOMMUEXCPT_STATUS =     0xb500UL,
	IOMMUPAGE_PADDR1 =      0xb580UL,
	IOMMUPAGE_PADDR2 =      0xb600UL,
	IOMMUPAGE_PADDR3 =      0xb680UL,
};

/* RC */
enum {
	RC_PRIMARY_BUS =	0x300UL,
};
/*--------------------------------------------------------------------------*/
#endif   /* _ASM_SW64_CHIP_QEMU_IO_H */
